# INTEL CONFIDENTIAL
# Copyright 2014 2015 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all
# documents related to the source code ("Material") are
# owned by Intel Corporation or its suppliers or licensors.
# Title to the Material remains with Intel Corporation or
# its suppliers and licensors. The Material may contain trade
# secrets and proprietary and confidential information of Intel
# Corporation and its suppliers and licensors, and is protected
# by worldwide copyright and trade secret laws and treaty
# provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you
# by disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.
from lib.diags import *
from lib.hw import HwError

def decode_help():
    return {"synopsis":
            "decode <reg name> <value>",
            "description":
            "Decode a value as if it was the value of register <reg name>. ",
            "arguments":
            [[("<reg name>", "The CSR which will serve as the template"),
             ("<value>", "The specific hexidecimal value to decode")]]}

def decode_completer(obj, text, line, start_index, end_index):
    return register_name_complete(obj.hw, text, line, start_index, end_index)

def decode_register(obj, line, output):
    if not line:
        obj.error("Invalid decode command")
        return False
    cmdline = line.split()
    if len(cmdline) < 2:
        obj.error("Invalid number of arguments to 'decode' command")
        return False
    
    if not is_number(cmdline[1]):
        obj.error("Second argument must be a numerical value")
        return False

    reginfo = []
    if not validate_register_spec(cmdline[0], reginfo, obj.hw,
                                  obj.hw.get_register_names() +
                                  obj.hw.get_config_register_names()):
        obj.error(reginfo[0])
        return False

    if is_hex(cmdline[1]): value = int(cmdline[1], 16)
    else: value = int(cmdline[1])

    for csr in reginfo:
        reg, cs, ce, rs, re, field = csr
        try: addr = obj.hw.addressof(reg)
        except HwError, e:
            obj.error(e)
            return False

        decoded = obj.hw.decode_register(reg, value)
        if field: show_fields = [getattr(decoded, field)]
        else: show_fields = decoded.fields
        output.append(obj.format_register(reg, -1, -1, addr, value,
                                          show_fields) + "\n")
    return False
    
__commands__ = [("decode", decode_register, decode_completer, decode_help)]
