# INTEL CONFIDENTIAL
# Copyright 2014 2015 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all
# documents related to the source code ("Material") are
# owned by Intel Corporation or its suppliers or licensors.
# Title to the Material remains with Intel Corporation or
# its suppliers and licensors. The Material may contain trade
# secrets and proprietary and confidential information of Intel
# Corporation and its suppliers and licensors, and is protected
# by worldwide copyright and trade secret laws and treaty
# provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you
# by disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.
from lib.diags import *
import lib.structures as libstruct

libstruct.define_struct("PBC",
                        #  Field name          Bitwidth
                        [("PbcLengthDWs",         12),
                         ("PbcVL",                4),
                         ("Reserved1",            6),
                         ("PbcFecn",              1),
                         ("PbcTestBadIcrc",       1),
                         ("PbcInsertBypassIcrc",  1),
                         ("PbcCreditReturn",      1),
                         ("PbcInsertHcrc",        2),
                         ("PbcPacketBypass",      1),
                         ("PbcTestEbp",           1),
                         ("PbcDcInfo",            1),
                         ("PbcIntr",              1),
                         ("PbcStaticRateControl", 16),
                         ("Reserved2",            16)],
                        libstruct.STRUCT_ENDIAN_NATIVE)

libstruct.define_struct("RHF",
                        [("PktLen",               12),
                         ("RcvType",              3),
                         ("UseEgrBfr",            1),
                         ("EgrIndex",             11),
                         ("DcInfo",               1),
                         ("RcvSeq",               4),
                         ("EgrOffset",            12),
                         ("HdrqOffset",           9),
                         ("KHdrLenErr",           1),
                         ("DcUncErr",             1),
                         ("DcErr",                1),
                         ("RcvTypeErr",           3),
                         ("TIDErr",               1),
                         ("LenErr",               1),
                         ("EccErr",               1),
                         ("Reserved",             1),
                         ("ICRCErr",              1)],
                        libstruct.STRUCT_ENDIAN_NATIVE)

libstruct.define_struct("LRH",
                        [("SC",                   4),
                         ("LVer",                 4),
                         ("SL",                   4),
                         ("Resv2",                2),
                         ("LNH",                  2),
                         ("DLID",                 16),
                         ("Resv4",                4),
                         ("PktLen",               12),
                         ("SLID",                 16)],
                        libstruct.STRUCT_ENDIAN_BIG)

libstruct.define_struct("BTH",
                        [("OpCode",               8),
                         ("SE",                   1),
                         ("M",                    1),
                         ("PadCnt",               2),
                         ("TVer",                 4),
                         ("P_KEY",                16),
                         ("F",                    1),
                         ("B",                    1),
                         ("Resv6",                6),
                         ("DestQP",               24),
                         ("A",                    1),
                         ("Resv7",                7),
                         ("PSN",                  24)],
                        libstruct.STRUCT_ENDIAN_BIG)

libstruct.define_struct("BTH31",
                        [("OpCode",               8),
                         ("SE",                   1),
                         ("M",                    1),
                         ("PadCnt",               2),
                         ("TVer",                 4),
                         ("P_KEY",                16),
                         ("F",                    1),
                         ("B",                    1),
                         ("Resv6",                6),
                         ("DestQP",               24),
                         ("A",                    1),
                         ("PSN",                  31)],
                        libstruct.STRUCT_ENDIAN_BIG)

libstruct.define_struct("KDETH",
                        [("Offset",               15),
                         ("OM",                   1),
                         ("TID",                  10),
                         ("TIDCtrl",              2),
                         ("Intr",                 1),
                         ("SH",                   1),
                         ("KVer",                 2),
                         ("J_KEY",                16),
                         ("HCRC_High",            8),
                         ("HCRC_Low",             8)],
                        libstruct.STRUCT_ENDIAN_LITTLE)

libstruct.define_struct("SDMADESC",
                        [("SDmaPhyAddr[31-0]",    32),
                         ("SDmaPhyAddr[47-32]",   16),
                         ("SDmaByteCount",        14),
                         ("SDmaLastDesc",         1),
                         ("SDmaFirstDesc",        1),
                         ("SDmaHeadToHost",       1),
                         ("SDmaIntReq",           1),
                         ("SDmaGeneration",       2),
                         ("SDmaHeaderDWs",        4),
                         ("SDmaHeaderIndex",      5),
                         ("SDmaHeaderMode",       3),
                         ("Reserved",             16),
                         ("SDmaHeaderUpdate1",    32)],
                        libstruct.STRUCT_ENDIAN_NATIVE,
                        wordsize=4)
libstruct.define_struct("SDMAAHG",
                        [("SDmaHeaderUpdateFieldValue",    16),
                         ("SDmaHeaderUpdateIndex"     ,    4),
                         ("SDmaHeaderUpdateFieldLength",   4),
                         ("SDmaHeaderUpdateFieldStartBit", 5),
                         ("SDmaHeaderUpdateFieldReserved", 2),
                         ("SDmaHeaderUpdateEnable",        1)],
                        libstruct.STRUCT_ENDIAN_NATIVE)

def header_parse_help():
    return {"synopsis": "header <header spec> <header value> [<header value>]",
            "description":
            "The 'header' command decodes a hex bitstream (or set of " + \
            "double-word values representing a particular STL header.\n\n" + \
            "The value(s) to be used for decoding can be a HEX bitstream - " + \
            "a string of HEX digits (starting with \"0x\") representing a " + \
            "portion or the entire header, or a set of double-word-sized " + \
            "HEX digits (each starting with \"0x\") representing each of " + \
            "the double-words of the header.\n\nThe '<header spec>' format " + \
            "consists of the header name, optionally followed by an double-word " + \
            "index or double-word range. If the index/range part is omitted, " + \
            "the range is assumed to be the entire header.\n\nIn the case where " + \
            "the header range is the entire header and the header input values " + \
            "are not long enough to cover the entire range, the input values " + \
            "are zero-padded according to the byte order of the header to " + \
            "provide sufficient bytes to cover the range. If the header is " + \
            "Little Endian, the zero-padding is added to the beginning of the " + \
            "bitstream, if Big Endian - to the end.\n\n"
            }

def parse_header_spec(header):
    if "[" not in header: name, start, stop = header, 0, -1
    else:
        if not header.endswith("]"): return (None, -1, -1)
        name = header[:header.index("[")]
        hrange = header[header.index("[")+1:-1]
        if "-" in hrange: hrange = hrange.split("-")
        else: hrange = hrange, hrange
        try: start, stop = map(int, hrange)
        except ValueError: return (name, -1, -1)
    return (name, start, stop)

def header_completion(obj, text, line, sidx, eidx):
    allheaders = libstruct.defined_structs()
    if text: return find_matches(text, allheaders)
    else: return allheaders

def header_process(obj, line, output):
    words = line.split()
    if not words:
        obj.error("Invalid command")
        return False

    if len(words) < 2:
        obj.error("'header' command requires at least 2 arguments")
        return False
    hname, start, end = parse_header_spec(words[0])
    if not hname:
        obj.error("Unparsable header spec: %s" % words[0])
        return False
    inst = libstruct.get_struct_by_name(hname)
    if not inst:
        obj.error("Could not find structure definitions for '%s'" % hname)
        return False
    elems = libstruct.struct_elems(inst)
    elemname = libstruct.struct_elem_name(inst).capitalize()
    elemsize = libstruct.struct_elem_size(inst)
    if start == -1:
        obj.error("Invalid range specification for structure/header %s" % hname)
        return False
    elif start >= elems:
        obj.error("Structure/header %s has only %d %ss" % (hname, elems, elemname))
        return False
    if end == -1: end = elems - 1
    elif end >= elems:
        obj.error("Structure/header %s has only %d %ss" % (hname, elems, elemname))
        return False
    elems_in_range = (end - start + 1)
    if len(words[1:]) > 1:
        bytes = ""
        for word in words[1:]:
            if not is_hex(word):
                obj.error("Structure/header value arguments must be HEX values")
                return False
            bytes += "0" * ((elemsize * 2) - len(word[2:])) + word[2:]
    else:
        if not is_hex(words[1]):
            obj.error("Header value requires a HEX stream")
            return False
        bytes = words[1][2:]

    # Make sure that we have enough to cover the range
    if len(libstruct.str2elems(inst, bytes)) < elems_in_range:
            obj.error("Insufficient number of %s values to cover range" % elemname)
            return False

    bytes_in_range = elems_in_range * elemsize
    # zefo-fill the begining of the byte stream to match the legth of the
    # requested element range
    bytes = bytes.zfill(bytes_in_range * 2)
    # split the byte stream into bytes and discard anything past the struct size
    bytes = libstruct.str2elems(inst, bytes)[:elems]
    # This is an empty bytestream to cover the entire header length. We'll
    # fill in the appropriate bytes
    bytestream = ["0" * (elemsize * 2) ] * elems
    if len(bytes) > elems_in_range:
        lendiff = elems - len(bytes)
        #if inst.is_le():
        #    for idx in range(elems-1, lendiff-1, -1):
        #        bytestream[idx] = bytes[idx-lendiff]
        #else:
        for idx in range(len(bytes)): bytestream[idx] = bytes[idx]
	# TODO: Fix me - two reverses can be eliminated
	if len(words[1:]) == 1 and hname in ("RHF","PBC","SDMADESC") : bytestream.reverse()
    else:
        for idx in range(start, end+1): bytestream[idx] = bytes[idx-start]
	# TODO: Fix me
	if len(words[1:]) > 1 and hname in ("RHF","PBC","SDMADESC") : bytestream.reverse()
    bytemap = [(x >= start and x <= end) and 1 or 0 for x in range(elems)]
    # Fix the addressing order so it is more intuitive.
    # Filter the elements based on when the user specified that element in the range
    bytestream = [bytemap[x] and bytestream[x] or ("0" * (elemsize * 2)) \
                  for x in range(elems)]
    # TODO: Fix me
    if len(bytes) > elems_in_range and hname in ("RHF","PBC","SDMADESC") : bytestream.reverse()
    try: inst = libstruct.struct_set_from_instance(inst, "".join(bytestream))
    except libstruct.StructError, e:
        obj.error("HEADER ERROR: " + str(e))
        return False
    fields = inst.get_field_names()
    output += [(hname, "0x"+"".join(bytestream)), SPACER]
    for f in fields:
        value = inst.get_field_by_name(f)
        if not value and not obj.config.show_zero_bitfield_values: continue
        output.append((hname+"."+f+":", "%#x" % value))
    return False

__commands__ = [("header", header_process, header_completion, header_parse_help)]
