# INTEL CONFIDENTIAL
# Copyright 2014 2015 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all
# documents related to the source code ("Material") are
# owned by Intel Corporation or its suppliers or licensors.
# Title to the Material remains with Intel Corporation or
# its suppliers and licensors. The Material may contain trade
# secrets and proprietary and confidential information of Intel
# Corporation and its suppliers and licensors, and is protected
# by worldwide copyright and trade secret laws and treaty
# provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you
# by disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.
import logging
import sys

_levelNames = logging._levelNames
getLevelName = logging.getLevelName
addLevelName = logging.addLevelName

gdict = globals()
for key in _levelNames.keys():
    if str(key).isalnum() and str(key)[0].isalpha():
        gdict[key] = logging._levelNames[key]

# Python version prior to 2.7 do not have the NullHandler
# class defined so we have to define it instead.
if not hasattr(logging, "NullHandler"):
    class NullHandler(logging.Handler):
        def handler(self, record): pass
        def emit(self, record): pass
        def createLock(self): self.lock = None
    logging.NullHandler = NullHandler

def initLogging(level=INFO, filename=None):
    logging.basicConfig(filename=filename,
                        format="[%(asctime)s] %(name)s %(levelname)s: %(message)s",
                        level=level,
                        stream=sys.stderr)

def initModuleLogging(logger):
    module = logger.split(".")[-1]
    logger = logging.getLogger(module.upper())
    # Add the NullHandler to prevent the "No handlers could be found..."
    # warning displayed when the logging module has not been initialized.
    logger.addHandler(logging.NullHandler())
    return logger

