# INTEL CONFIDENTIAL
# Copyright 2014-2016 Intel Corporation All Rights Reserved.
#
# The source code contained or described herein and all
# documents related to the source code ("Material") are
# owned by Intel Corporation or its suppliers or licensors.
# Title to the Material remains with Intel Corporation or
# its suppliers and licensors. The Material may contain trade
# secrets and proprietary and confidential information of Intel
# Corporation and its suppliers and licensors, and is protected
# by worldwide copyright and trade secret laws and treaty
# provisions. No part of the Material may be used, copied,
# reproduced, modified, published, uploaded, posted, transmitted,
# distributed, or disclosed in any way without Intel's prior express
# written permission.
#
# No license under any patent, copyright, trade secret or other
# intellectual property right is granted to or conferred upon you
# by disclosure or delivery of the Materials, either expressly, by
# implication, inducement, estoppel or otherwise. Any license under
# such intellectual property rights must be express and approved by
# Intel in writing.
#
# Unless otherwise agreed by Intel in writing, you may not remove or
# alter this notice or any other notice embedded in Materials by Intel
# or Intel's suppliers or licensors in any way.
import signal
import subprocess
import re

__device_regexp = re.compile(r'^(?:[0-9a-fA-F]+:)?[0-9a-fA-F]{2,}:[0-1][0-9a-fA-F]\.[0-7]+$')
__setpci_exe = None

try:
    import os
    for __d in os.environ.get("PATH", "").split(':'):
        __p = os.path.join(__d, "setpci")
        if os.access(__p, os.X_OK):
            __setpci_exe = __p
            break
    del __d, __p, os
except Exception:
    pass

def proc(cmd):
    return subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE, close_fds=True, shell=False)

def validate_device(device):
    return True if __device_regexp.match(device) else False

def enable_device(device):
    if not device:
        return False

    path = "/sys/bus/pci/devices/%s/enable" % device
    try:
        with open(path, 'w') as fd:
            fd.write("1")
        return True
    except IOError:
        return False

def read_register(device, address):
    if not __setpci_exe or not isinstance(address, (int, long)) or \
       not validate_device(device): return False
    _p = proc([__setpci_exe, "-s %s" % device, "0x%x.L" % address])
    ret = _p.wait()
    if ret == 0:
        result = _p.stdout.readline()
        try: ret = int(result, 16)
        except ValueError: ret = False
        return ret
    return False

def write_register(device, address, value):
    if not __setpci_exe or not isinstance(address, (int, long)) or \
       not isinstance(value, (int, long)) or \
       not validate_device(device): return False
    _p = proc([__setpci_exe, "-s %s" % device, "0x%x.L=0x%x" % (address, value)])
    ret = _p.wait()
    if ret == 0: return True
    return False
